from .decentraleyes_addon_settings import DecentraleyesAddonSettings
from .foxy_proxy_addon_settings import FoxyProxyAddonSettings
from .https_everywhere_addon_settings import HttpsEverywhereAddonSettings
from .noscript_addon_settings import NoscriptAddonSettings
from .privacy_badger_addon_settings import PrivacyBadgerAddonSettings
from .ublock_origin_addon_settings import UBlockOriginAddonSettings

from .builtin_addon_install_settings import BuiltinAddonInstallSettings
from .builtin_addon_install_settings.base_addon_install_settings import BaseAddonInstallSettings